package com.app.yourradioapp.activities;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.ProgressBar;

import androidx.appcompat.app.AppCompatActivity;

import com.android.volley.Request;
import com.android.volley.toolbox.JsonObjectRequest;
import com.app.yourradioapp.Config;
import com.app.yourradioapp.R;
import com.app.yourradioapp.utils.AdsPref;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.SharedPref;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;

import org.json.JSONException;

import static com.app.yourradioapp.utils.Constant.THEME_DARK;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;

public class ActivitySplash extends AppCompatActivity {

    ProgressBar progressBar;
    AdsPref adsPref;
    ThemePref themePref;
    SharedPref sharedPref;
    ImageView img_splash;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Tools.getTheme(this);
        //AudienceNetworkInitializeHelper.initialize(this);
        //AdSettings.setTestMode(TEST_MODE_FAN);
        //AdSettings.setIntegrationErrorMode(INTEGRATION_ERROR_CRASH_DEBUG_MODE);
        setContentView(R.layout.activity_splash);

        sharedPref = new SharedPref(this);
        themePref = new ThemePref(this);

        adsPref = new AdsPref(this);

        img_splash = findViewById(R.id.img_splash);
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            img_splash.setImageResource(R.drawable.splash_light);
        } else if (themePref.getCurrentTheme() == THEME_DARK) {
            img_splash.setImageResource(R.drawable.splash_dark);
        } else {
            img_splash.setImageResource(R.drawable.splash_primary);
        }

        progressBar = findViewById(R.id.progressBar);
        progressBar.setVisibility(View.VISIBLE);

        requestAds();

    }

    private void requestAds() {

        JsonObjectRequest jsonObjReq = new JsonObjectRequest(Request.Method.GET, Constant.URL_ADS, null, response -> {

            try {
                final String ad_status = response.getString("ad_status");
                final String ad_type = response.getString("ad_type");
                final String admob_publisher_id = response.getString("admob_publisher_id");
                final String admob_app_id = response.getString("admob_app_id");
                final String admob_banner_unit_id = response.getString("admob_banner_unit_id");
                final String admob_interstitial_unit_id = response.getString("admob_interstitial_unit_id");
                final String admob_native_unit_id = response.getString("admob_native_unit_id");
                final String fan_banner_unit_id = response.getString("fan_banner_unit_id");
                final String fan_interstitial_unit_id = response.getString("fan_interstitial_unit_id");
                final String fan_native_unit_id = response.getString("fan_native_unit_id");
                final String startapp_app_id = response.getString("startapp_app_id");
                final int interstitial_ad_interval = response.getInt("interstitial_ad_interval");
                final int native_ad_interval = response.getInt("native_ad_interval");
                final int native_ad_index = response.getInt("native_ad_index");
                final String date_time = response.getString("date_time");

                if (date_time.equals(adsPref.getDateTime())) {
                    Log.d("AD_STATUS", "Ads Data is updated");
                } else {
                    adsPref.saveAds(
                            ad_status,
                            ad_type,
                            admob_publisher_id,
                            admob_app_id,
                            admob_banner_unit_id,
                            admob_interstitial_unit_id,
                            admob_native_unit_id,
                            fan_banner_unit_id,
                            fan_interstitial_unit_id,
                            fan_native_unit_id,
                            startapp_app_id,
                            interstitial_ad_interval,
                            native_ad_interval,
                            native_ad_index,
                            date_time
                    );
                    Log.d("AD_STATUS", "Ads Data is saved");
                }

                new Handler().postDelayed(() -> {
                    Intent intent = new Intent(getApplicationContext(), MainActivity.class);
                    startActivity(intent);
                    finish();
                }, Config.SPLASH_DURATION);


            } catch (JSONException e) {
                e.printStackTrace();
                new Handler().postDelayed(() -> {
                    Intent intent = new Intent(getApplicationContext(), MainActivity.class);
                    startActivity(intent);
                    finish();
                }, Config.SPLASH_DURATION);
                Log.d("INFO", "Do Nothing");
            }
        }, error -> {
            new Handler().postDelayed(() -> {
                Intent intent = new Intent(getApplicationContext(), MainActivity.class);
                startActivity(intent);
                finish();
            }, Config.SPLASH_DURATION);
            Log.d("INFO", "Do Nothing");
        }
        );

        MyApplication.getInstance().addToRequestQueue(jsonObjReq);
    }

}
