package com.app.yourradioapp.adapters;

import android.content.Context;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Filter;
import android.widget.Filterable;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.app.yourradioapp.Config;
import com.app.yourradioapp.R;
import com.app.yourradioapp.models.ItemCategory;
import com.balysv.materialripple.MaterialRippleLayout;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.List;

public class AdapterCategory extends RecyclerView.Adapter<AdapterCategory.MyViewHolder> implements Filterable {

    private Context context;
    private List<ItemCategory> categoryList;
    private List<ItemCategory> categoryListFiltered;
    private OnItemClickListener mOnItemClickListener;
    private boolean clicked;

    public interface OnItemClickListener {
        void onItemClick(View view, ItemCategory obj, int position);
    }

    public void setOnItemClickListener(final OnItemClickListener mItemOverflowClickListener) {
        this.mOnItemClickListener = mItemOverflowClickListener;
    }

    public static class MyViewHolder extends RecyclerView.ViewHolder {

        private TextView txt_category_name;
        private TextView txt_radio_count;
        private ImageView img_category;
        private MaterialRippleLayout rippleLayout;

        public MyViewHolder(View view) {
            super(view);

            rippleLayout = view.findViewById(R.id.ripple);
            txt_category_name = view.findViewById(R.id.txt_category_name);
            txt_radio_count = view.findViewById(R.id.txt_radio_count);
            img_category = view.findViewById(R.id.img_category);

        }
    }

    public AdapterCategory(Context context, List<ItemCategory> categoryList) {
        this.context = context;
        this.categoryList = categoryList;
        this.categoryListFiltered = categoryList;
    }

    @NonNull
    @Override
    public MyViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.lsv_item_category, parent, false);
        return new MyViewHolder(view);
    }

    @Override
    public void onBindViewHolder(MyViewHolder holder, final int position) {
        final ItemCategory category = categoryListFiltered.get(position);

        holder.txt_category_name.setText(category.getCategory_name());

        holder.txt_radio_count.setText(category.getRadio_count() + " " + context.getResources().getString(R.string.station));
        if (Config.DISPLAY_CHANNEL_COUNT_ON_CATEGORY_LIST) {
            holder.txt_radio_count.setVisibility(View.VISIBLE);
        } else {
            holder.txt_radio_count.setVisibility(View.GONE);
        }

        Picasso.with(context)
                .load(Config.ADMIN_PANEL_URL + "/upload/category/" + category.getCategory_image())
                .placeholder(R.drawable.ic_thumbnail)
                .into(holder.img_category);

        holder.rippleLayout.setOnClickListener(view -> {
            if (clicked) {
                return;
            }
            clicked = true;
            new Handler().postDelayed(() -> clicked = false, 2000);

            if (mOnItemClickListener != null) {
                mOnItemClickListener.onItemClick(view, category, position);
            }
        });

    }

    @Override
    public int getItemCount() {
        return categoryListFiltered.size();
    }

    @Override
    public Filter getFilter() {
        return new Filter() {
            @Override
            protected FilterResults performFiltering(CharSequence charSequence) {
                String charString = charSequence.toString();
                if (charString.isEmpty()) {
                    categoryListFiltered = categoryList;
                } else {
                    List<ItemCategory> filteredList = new ArrayList<>();
                    for (ItemCategory row : categoryList) {
                        if (row.getCategory_name().toLowerCase().contains(charString.toLowerCase())) {
                            filteredList.add(row);
                        }
                    }
                    categoryListFiltered = filteredList;
                }

                FilterResults filterResults = new FilterResults();
                filterResults.values = categoryListFiltered;
                return filterResults;
            }

            @SuppressWarnings("unchecked")
            @Override
            protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
                categoryListFiltered = (ArrayList<ItemCategory>) filterResults.values;
                notifyDataSetChanged();
            }
        };
    }

}
