package com.app.yourradioapp.adapters;

import android.app.Activity;
import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.Filter;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.Config;
import com.app.yourradioapp.R;
import com.app.yourradioapp.models.ItemRadio;
import com.app.yourradioapp.utils.AdsPref;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.NativeTemplateStyle;
import com.app.yourradioapp.utils.TemplateView;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;
import com.balysv.materialripple.MaterialRippleLayout;
import com.facebook.ads.AdError;
import com.facebook.ads.AdOptionsView;
import com.facebook.ads.NativeAd;
import com.facebook.ads.NativeAdLayout;
import com.facebook.ads.NativeAdListener;
import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdLoader;
import com.google.android.gms.ads.LoadAdError;
import com.google.android.gms.ads.formats.MediaView;
import com.squareup.picasso.Picasso;
import com.startapp.sdk.ads.nativead.NativeAdDetails;
import com.startapp.sdk.ads.nativead.NativeAdPreferences;
import com.startapp.sdk.ads.nativead.StartAppNativeAd;
import com.startapp.sdk.adsbase.Ad;
import com.startapp.sdk.adsbase.adlisteners.AdEventListener;

import java.util.ArrayList;
import java.util.List;

import static com.app.yourradioapp.utils.Constant.ADMOB;
import static com.app.yourradioapp.utils.Constant.AD_STATUS_ON;
import static com.app.yourradioapp.utils.Constant.FAN;
import static com.app.yourradioapp.utils.Constant.STARTAPP;
import static com.app.yourradioapp.utils.Constant.STARTAPP_IMAGE_MEDIUM;
import static com.app.yourradioapp.utils.Constant.THEME_DARK;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;
import static com.app.yourradioapp.utils.Constant.THEME_PRIMARY;

public class AdapterRadio extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private final int VIEW_ITEM = 0;
    private final int VIEW_HEAD = 1;
    private ArrayList<ItemRadio> arrayList;
    private Context context;
    private ArrayList<ItemRadio> filteredArrayList;
    private NameFilter filter;
    private Tools tools;
    private OnItemClickListener mOnItemClickListener;
    private OnItemOverflowClickListener mOnItemOverflowClickListener;
    private StartAppNativeAd startAppNativeAd;
    private NativeAdDetails nativeAdDetails = null;

    public interface OnItemClickListener {
        void onItemClick(View view, ItemRadio obj, int position);
    }

    public void setOnItemClickListener(final OnItemClickListener mItemClickListener) {
        this.mOnItemClickListener = mItemClickListener;
    }

    public interface OnItemOverflowClickListener {
        void onItemOverflowClick(View view, ItemRadio obj, int position);
    }

    public void setOnItemOverflowClickListener(final OnItemOverflowClickListener mItemOverflowClickListener) {
        this.mOnItemOverflowClickListener = mItemOverflowClickListener;
    }

    public class HeadingViewHolder extends RecyclerView.ViewHolder {

        private ImageView img_radio;
        private ImageButton img_overflow;
        private TextView txt_radio, txt_category;
        private MaterialRippleLayout rippleLayout;

        LinearLayout lyt_native_ad;
        TemplateView admob_native_template;
        MediaView admob_media_view;
        TextView primary;
        TextView secondary;

        private NativeAd nativeAd;
        RelativeLayout lyt_fan_native;
        private NativeAdLayout nativeAdLayout;
        private LinearLayout nativeAdView;

        RelativeLayout lyt_startapp_native;
        ImageView startapp_native_image;
        TextView startapp_native_title;
        TextView startapp_native_description;
        Button startapp_native_button;

        private HeadingViewHolder(View view) {
            super(view);
            rippleLayout = view.findViewById(R.id.ripple);
            txt_radio = view.findViewById(R.id.txt_radio);
            txt_category = view.findViewById(R.id.txt_category);
            img_radio = view.findViewById(R.id.img_radio);
            img_overflow = view.findViewById(R.id.img_overflow);

            //admob native ad
            lyt_native_ad = view.findViewById(R.id.lyt_native_ad);
            admob_native_template = view.findViewById(R.id.native_template);
            admob_media_view = view.findViewById(R.id.media_view);
            primary = view.findViewById(R.id.primary);
            secondary = view.findViewById(R.id.secondary);

            //fan native ad
            lyt_fan_native = view.findViewById(R.id.lyt_fan_native);
            nativeAdLayout = view.findViewById(R.id.native_ad_container);

            //startapp native ad
            lyt_startapp_native = view.findViewById(R.id.lyt_startapp_native);
            startapp_native_image = view.findViewById(R.id.startapp_native_image);
            startapp_native_title = view.findViewById(R.id.startapp_native_title);
            startapp_native_description = view.findViewById(R.id.startapp_native_description);
            startapp_native_button = view.findViewById(R.id.startapp_native_button);
            startapp_native_button.setOnClickListener(v1 -> itemView.performClick());

        }

        private void bindAdMobNativeAdView() {
            final ThemePref themePref = new ThemePref(context);
            final AdsPref adsPref = new AdsPref(context);
            AdLoader adLoader = new AdLoader.Builder(context, adsPref.getAdMobNativeId())
                    .forUnifiedNativeAd(unifiedNativeAd -> {
                        if (themePref.getCurrentTheme() == THEME_LIGHT || themePref.getCurrentTheme() == THEME_PRIMARY) {
                            ColorDrawable colorDrawable = new ColorDrawable(ContextCompat.getColor(context, R.color.colorLight));
                            NativeTemplateStyle styles = new NativeTemplateStyle.Builder().withMainBackgroundColor(colorDrawable).build();
                            admob_native_template.setStyles(styles);
                            primary.setBackgroundResource(R.color.colorLight);
                            secondary.setBackgroundResource(R.color.colorLight);
                        } else if (themePref.getCurrentTheme() == THEME_DARK) {
                            ColorDrawable colorDrawable = new ColorDrawable(ContextCompat.getColor(context, R.color.colorBackgroundDark));
                            NativeTemplateStyle styles = new NativeTemplateStyle.Builder().withMainBackgroundColor(colorDrawable).build();
                            admob_native_template.setStyles(styles);
                            primary.setBackgroundResource(R.color.colorBackgroundDark);
                            secondary.setBackgroundResource(R.color.colorBackgroundDark);
                        }
                        admob_media_view.setImageScaleType(ImageView.ScaleType.CENTER_CROP);
                        admob_native_template.setNativeAd(unifiedNativeAd);
                    }).withAdListener(new AdListener() {
                        @Override
                        public void onAdLoaded() {
                            super.onAdLoaded();
                            admob_native_template.setVisibility(View.VISIBLE);
                            lyt_native_ad.setVisibility(View.VISIBLE);
                        }

                        @Override
                        public void onAdFailedToLoad(LoadAdError adError) {
                            admob_native_template.setVisibility(View.GONE);
                            lyt_native_ad.setVisibility(View.GONE);
                        }
                    })
                    .build();
            adLoader.loadAd(Tools.getAdRequest((Activity) context));
        }

        private void bindFanNativeAdView() {
            final AdsPref adsPref = new AdsPref(context);
            final ThemePref themePref = new ThemePref(context);
            if (BuildConfig.DEBUG) {
                nativeAd = new NativeAd(context, "IMG_16_9_APP_INSTALL#" + adsPref.getFanNativeUnitId());
            } else {
                nativeAd = new NativeAd(context, adsPref.getFanNativeUnitId());
            }
            NativeAdListener nativeAdListener = new NativeAdListener() {
                @Override
                public void onMediaDownloaded(com.facebook.ads.Ad ad) {

                }

                @Override
                public void onError(com.facebook.ads.Ad ad, AdError adError) {

                }

                @Override
                public void onAdLoaded(com.facebook.ads.Ad ad) {
                    // Race condition, load() called again before last ad was displayed
                    if (getAdapterPosition() % adsPref.getNativeAdInterval() == adsPref.getNativeAdIndex()) {
                        lyt_fan_native.setVisibility(View.VISIBLE);
                        if (nativeAd == null || nativeAd != ad) {
                            return;
                        }
                        // Inflate Native Ad into Container
                        //inflateAd(nativeAd);
                        nativeAd.unregisterView();
                        // Add the Ad view into the ad container.
                        LayoutInflater inflater = LayoutInflater.from(context);
                        // Inflate the Ad view.  The layout referenced should be the one you created in the last step.

                        nativeAdView = (LinearLayout) inflater.inflate(R.layout.gnt_fan_small_template, nativeAdLayout, false);

                        nativeAdLayout.addView(nativeAdView);

                        // Add the AdOptionsView
                        LinearLayout adChoicesContainer = nativeAdView.findViewById(R.id.ad_choices_container);
                        AdOptionsView adOptionsView = new AdOptionsView(context, nativeAd, nativeAdLayout);
                        adChoicesContainer.removeAllViews();
                        adChoicesContainer.addView(adOptionsView, 0);

                        // Create native UI using the ad metadata.
                        TextView nativeAdTitle = nativeAdView.findViewById(R.id.native_ad_title);
                        com.facebook.ads.MediaView nativeAdMedia = nativeAdView.findViewById(R.id.native_ad_media);
                        TextView nativeAdSocialContext = nativeAdView.findViewById(R.id.native_ad_social_context);
                        TextView nativeAdBody = nativeAdView.findViewById(R.id.native_ad_body);
                        TextView sponsoredLabel = nativeAdView.findViewById(R.id.native_ad_sponsored_label);
                        Button nativeAdCallToAction = nativeAdView.findViewById(R.id.native_ad_call_to_action);
                        LinearLayout ad_unit = nativeAdView.findViewById(R.id.ad_unit);

                        if (themePref.getCurrentTheme() == THEME_LIGHT || themePref.getCurrentTheme() == THEME_PRIMARY) {
                            nativeAdTitle.setBackgroundResource(R.color.colorLight);
                        } else if (themePref.getCurrentTheme() == THEME_DARK) {
                            nativeAdTitle.setBackgroundResource(R.color.colorBackgroundDark);
                        }

                        // Set the Text.
                        nativeAdTitle.setText(nativeAd.getAdvertiserName());
                        nativeAdBody.setText(nativeAd.getAdBodyText());
                        nativeAdSocialContext.setText(nativeAd.getAdSocialContext());
                        nativeAdCallToAction.setVisibility(nativeAd.hasCallToAction() ? View.VISIBLE : View.INVISIBLE);
                        nativeAdCallToAction.setText(nativeAd.getAdCallToAction());
                        sponsoredLabel.setText(nativeAd.getSponsoredTranslation());

                        // Create a list of clickable views
                        List<View> clickableViews = new ArrayList<>();
                        clickableViews.add(nativeAdTitle);
                        clickableViews.add(ad_unit);
                        clickableViews.add(nativeAdCallToAction);

                        // Register the Title and CTA button to listen for clicks.
                        nativeAd.registerViewForInteraction(nativeAdView, nativeAdMedia, clickableViews);
                    } else {
                        lyt_fan_native.setVisibility(View.GONE);
                    }
                }

                @Override
                public void onAdClicked(com.facebook.ads.Ad ad) {

                }

                @Override
                public void onLoggingImpression(com.facebook.ads.Ad ad) {

                }
            };

            NativeAd.NativeLoadAdConfig loadAdConfig = nativeAd.buildLoadAdConfig().withAdListener(nativeAdListener).build();
            nativeAd.loadAd(loadAdConfig);
        }

        private void bindStartAppNativeAdView() {
            startAppNativeAd = new StartAppNativeAd(context);
            final AdsPref adsPref = new AdsPref(context);
            final ThemePref themePref = new ThemePref(context);
            NativeAdPreferences nativePrefs = new NativeAdPreferences()
                    .setAdsNumber(3)
                    .setAutoBitmapDownload(true)
                    .setPrimaryImageSize(STARTAPP_IMAGE_MEDIUM);
            AdEventListener adListener = new AdEventListener() {
                @Override
                public void onReceiveAd(Ad arg0) {
                    if (getAdapterPosition() % adsPref.getNativeAdInterval() == adsPref.getNativeAdIndex()) {
                        ArrayList<NativeAdDetails> nativeAdsList = startAppNativeAd.getNativeAds();
                        if (nativeAdsList.size() > 0) {
                            nativeAdDetails = nativeAdsList.get(0);
                        }
                        if (nativeAdDetails != null) {
                            startapp_native_image.setImageBitmap(nativeAdDetails.getImageBitmap());

                            if (themePref.getCurrentTheme() == THEME_LIGHT || themePref.getCurrentTheme() == THEME_PRIMARY) {
                                startapp_native_title.setBackgroundResource(R.color.colorLight);
                                startapp_native_description.setBackgroundResource(R.color.colorLight);
                            } else if (themePref.getCurrentTheme() == THEME_DARK) {
                                startapp_native_title.setBackgroundResource(R.color.colorBackgroundDark);
                                startapp_native_description.setBackgroundResource(R.color.colorBackgroundDark);
                            }

                            startapp_native_title.setText(nativeAdDetails.getTitle());
                            startapp_native_description.setText(nativeAdDetails.getDescription());
                            startapp_native_button.setText(nativeAdDetails.isApp() ? "Install" : "Open");
                            nativeAdDetails.registerViewForInteraction(itemView);
                        }
                        lyt_startapp_native.setVisibility(View.VISIBLE);
                    } else {
                        lyt_startapp_native.setVisibility(View.GONE);
                    }
                }

                @Override
                public void onFailedToReceiveAd(Ad arg0) {
                    lyt_startapp_native.setVisibility(View.GONE);
                }
            };
            startAppNativeAd.loadAd(nativePrefs, adListener);
        }

    }

    public static class OriginalViewHolder extends RecyclerView.ViewHolder {

        private ImageView img_radio;
        private ImageButton img_overflow;
        private TextView txt_radio, txt_category;
        private MaterialRippleLayout rippleLayout;

        private OriginalViewHolder(View view) {
            super(view);
            rippleLayout = view.findViewById(R.id.ripple);
            txt_radio = view.findViewById(R.id.txt_radio);
            txt_category = view.findViewById(R.id.txt_category);
            img_radio = view.findViewById(R.id.img_radio);
            img_overflow = view.findViewById(R.id.img_overflow);
        }

    }

    public AdapterRadio(Context context, ArrayList<ItemRadio> arrayList) {
        this.arrayList = arrayList;
        this.context = context;
        filteredArrayList = arrayList;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh;
        if (viewType == VIEW_ITEM) {
            View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.lsv_item_radio, parent, false);
            vh = new OriginalViewHolder(v);
        } else if (viewType == VIEW_HEAD) {
            View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.lsv_item_native_ad, parent, false);
            vh = new HeadingViewHolder(v);
        } else {
            View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.lsv_item_radio, parent, false);
            vh = new OriginalViewHolder(v);
        }
        return vh;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, final int position) {

        if (holder instanceof OriginalViewHolder) {

            OriginalViewHolder vItem = (OriginalViewHolder) holder;
            final ItemRadio itemRadio = arrayList.get(position);

            vItem.txt_radio.setText(itemRadio.getRadio_name());
            vItem.txt_category.setText(itemRadio.getCategory_name());

            Picasso.with(context)
                    .load(Config.ADMIN_PANEL_URL + "/upload/" + itemRadio.getRadio_image())
                    .placeholder(R.drawable.ic_thumbnail)
                    .into(vItem.img_radio);

            vItem.img_overflow.setOnClickListener(view -> {
                if (mOnItemOverflowClickListener != null) {
                    mOnItemOverflowClickListener.onItemOverflowClick(view, itemRadio, position);
                }
            });

            vItem.rippleLayout.setOnClickListener(view -> {
                if (mOnItemClickListener != null) {
                    mOnItemClickListener.onItemClick(view, itemRadio, position);
                }
            });

        } else {

            HeadingViewHolder vItem = (HeadingViewHolder) holder;
            final ItemRadio itemRadio = arrayList.get(position);

            final AdsPref adsPref = new AdsPref(context);
            int LIMIT_NATIVE_AD = (Constant.MAX_NUMBER_OF_NATIVE_AD_DISPLAYED * adsPref.getNativeAdInterval()) + adsPref.getNativeAdIndex();
            if (adsPref.getAdStatus().equals(AD_STATUS_ON)) {
                for (int i = adsPref.getNativeAdIndex(); i < LIMIT_NATIVE_AD; i += adsPref.getNativeAdInterval()) {
                    if (position == i) {
                        switch (adsPref.getAdType()) {
                            case ADMOB:
                                if (!adsPref.getAdMobNativeId().equals("0")) {
                                    vItem.bindAdMobNativeAdView();
                                }
                                break;
                            case FAN:
                                if (!adsPref.getFanNativeUnitId().equals("0")) {
                                    vItem.bindFanNativeAdView();
                                }
                                break;
                            case STARTAPP:
                                if (!adsPref.getStartappAppID().equals("0")) {
                                    vItem.bindStartAppNativeAdView();
                                }
                                break;
                        }
                    }
                }
            }

            vItem.txt_radio.setText(itemRadio.getRadio_name());
            vItem.txt_category.setText(itemRadio.getCategory_name());

            Picasso.with(context)
                    .load(Config.ADMIN_PANEL_URL + "/upload/" + itemRadio.getRadio_image())
                    .placeholder(R.drawable.ic_thumbnail)
                    .into(vItem.img_radio);

            vItem.img_overflow.setOnClickListener(view -> {
                if (mOnItemOverflowClickListener != null) {
                    mOnItemOverflowClickListener.onItemOverflowClick(view, itemRadio, position);
                }
            });

            vItem.rippleLayout.setOnClickListener(view -> {
                if (mOnItemClickListener != null) {
                    mOnItemClickListener.onItemClick(view, itemRadio, position);
                }
            });

        }

    }

    @Override
    public int getItemCount() {
        return arrayList.size();
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemViewType(int position) {
        if (arrayList.get(position) != null) {
            final AdsPref adsPref = new AdsPref(context);
            int LIMIT_NATIVE_AD = (Constant.MAX_NUMBER_OF_NATIVE_AD_DISPLAYED * adsPref.getNativeAdInterval()) + adsPref.getNativeAdIndex();
            for (int i = adsPref.getNativeAdIndex(); i < LIMIT_NATIVE_AD; i += adsPref.getNativeAdInterval()) {
                if (position == i) {
                    return VIEW_HEAD;
                }
            }
            return VIEW_ITEM;
        } else {
            return VIEW_ITEM;
        }
    }

    private class NameFilter extends Filter {

        @Override
        protected FilterResults performFiltering(CharSequence constraint) {

            constraint = constraint.toString().toLowerCase();
            FilterResults result = new FilterResults();
            if (constraint.toString().length() > 0) {
                ArrayList<ItemRadio> filteredItems = new ArrayList<>();

                for (int i = 0, l = filteredArrayList.size(); i < l; i++) {
                    String nameList = filteredArrayList.get(i).getRadio_name();
                    if (nameList.toLowerCase().contains(constraint))
                        filteredItems.add(filteredArrayList.get(i));
                }
                result.count = filteredItems.size();
                result.values = filteredItems;
            } else {
                synchronized (this) {
                    result.values = filteredArrayList;
                    result.count = filteredArrayList.size();
                }
            }
            return result;
        }

        @SuppressWarnings("unchecked")
        @Override
        protected void publishResults(CharSequence constraint, FilterResults results) {
            arrayList = (ArrayList<ItemRadio>) results.values;
            notifyDataSetChanged();
        }
    }

//    private Tools.onItemClickListener onItemClickListener = new Tools.onItemClickListener() {
//        @Override
//        public void onClick(int position, String type) {
//            if (tools.isNetworkAvailable()) {
//                Constant.item_radio.clear();
//                Constant.item_radio.addAll(arrayList);
//                ((MainActivity) context).clickPlay(position, true);
//            } else {
//                Toast.makeText(context, context.getResources().getString(R.string.internet_not_connected), Toast.LENGTH_SHORT).show();
//            }
//        }
//    };

}