package com.app.yourradioapp.adapters;

import android.content.Context;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.app.yourradioapp.Config;
import com.app.yourradioapp.R;
import com.app.yourradioapp.models.ItemSocial;
import com.balysv.materialripple.MaterialRippleLayout;
import com.squareup.picasso.Picasso;

import java.util.List;

public class AdapterSocial extends RecyclerView.Adapter<AdapterSocial.MyViewHolder> {

    private Context context;
    private List<ItemSocial> categoryList;
    private OnItemClickListener mOnItemClickListener;
    private boolean clicked;

    public interface OnItemClickListener {
        void onItemClick(View view, ItemSocial obj, int position);
    }

    public void setOnItemClickListener(final OnItemClickListener mItemOverflowClickListener) {
        this.mOnItemClickListener = mItemOverflowClickListener;
    }

    public static class MyViewHolder extends RecyclerView.ViewHolder {

        private TextView txt_name;
        private ImageView img_icon;
        private MaterialRippleLayout rippleLayout;

        public MyViewHolder(View view) {
            super(view);
            rippleLayout = view.findViewById(R.id.ripple);
            txt_name = view.findViewById(R.id.txt_name);
            img_icon = view.findViewById(R.id.img_icon);

        }
    }

    public AdapterSocial(Context context, List<ItemSocial> categoryList) {
        this.context = context;
        this.categoryList = categoryList;
    }

    @NonNull
    @Override
    public MyViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.lsv_item_social, parent, false);
        return new MyViewHolder(view);
    }

    @Override
    public void onBindViewHolder(MyViewHolder holder, final int position) {
        final ItemSocial p = categoryList.get(position);

        holder.txt_name.setText(p.getName());

        Picasso.with(context)
                .load(Config.ADMIN_PANEL_URL + "/upload/social/" + p.getIcon())
                .placeholder(R.drawable.ic_thumbnail)
                .into(holder.img_icon);

        holder.rippleLayout.setOnClickListener(view -> {
            if (clicked) {
                return;
            }
            clicked = true;
            new Handler().postDelayed(() -> clicked = false, 2000);

            if (mOnItemClickListener != null) {
                mOnItemClickListener.onItemClick(view, p, position);
            }
        });

    }

    @Override
    public int getItemCount() {
        return categoryList.size();
    }

}
