package com.app.yourradioapp.fragments;

import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RelativeLayout;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.android.volley.toolbox.JsonArrayRequest;
import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.Config;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MyApplication;
import com.app.yourradioapp.adapters.AdapterCategory;
import com.app.yourradioapp.models.ItemCategory;
import com.app.yourradioapp.utils.AdsPref;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.ItemOffsetDecoration;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;
import com.facebook.ads.Ad;
import com.facebook.ads.AdError;
import com.facebook.ads.InterstitialAdListener;
import com.facebook.shimmer.ShimmerFrameLayout;
import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.InterstitialAd;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.startapp.sdk.adsbase.StartAppAd;

import java.util.ArrayList;
import java.util.List;

import static com.app.yourradioapp.utils.Constant.ADMOB;
import static com.app.yourradioapp.utils.Constant.AD_STATUS_ON;
import static com.app.yourradioapp.utils.Constant.FAN;
import static com.app.yourradioapp.utils.Constant.STARTAPP;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;

public class FragmentCategory extends Fragment {

    private List<ItemCategory> categoryList;
    private AdapterCategory mAdapter;
    SwipeRefreshLayout swipeRefreshLayout = null;
    RecyclerView recyclerView;
    View lyt_empty, lyt_failed;
    Button btn_empty_retry, btn_failed_retry;
    RelativeLayout lyt_parent;
    RelativeLayout lyt_content;
    ShimmerFrameLayout lyt_shimmer;
    private InterstitialAd adMobInterstitialAd;
    private com.facebook.ads.InterstitialAd fanInterstitialAd;
    private StartAppAd startAppAd;
    private AdsPref adsPref;
    int counter = 1;
    Tools tools;
    ThemePref themePref;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_category, container, false);
        lyt_parent = view.findViewById(R.id.lyt_parent);

        setHasOptionsMenu(true);

        tools = new Tools(getActivity());
        lyt_shimmer = view.findViewById(R.id.shimmer_view_container);
        lyt_content = view.findViewById(R.id.lyt_content);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        swipeRefreshLayout.setColorSchemeResources(R.color.colorPrimary);

        themePref = new ThemePref(getActivity());
        adsPref = new AdsPref(getActivity());
        loadAdNetwork();

        lyt_empty = view.findViewById(R.id.lyt_empty);
        lyt_failed = view.findViewById(R.id.lyt_no_network);
        btn_empty_retry = view.findViewById(R.id.btn_empty_retry);
        btn_failed_retry = view.findViewById(R.id.btn_failed_retry);

        recyclerView = view.findViewById(R.id.recyclerView);
        categoryList = new ArrayList<>();
        mAdapter = new AdapterCategory(getActivity(), categoryList);

        recyclerView.setLayoutManager(new GridLayoutManager(getActivity(), Config.CATEGORY_COLUMN_COUNT));
        recyclerView.setHasFixedSize(true);
        if (getActivity() != null) {
            ItemOffsetDecoration itemDecoration = new ItemOffsetDecoration(getActivity(), R.dimen.item_offset);
            recyclerView.addItemDecoration(itemDecoration);
        }
        recyclerView.setAdapter(mAdapter);

        fetchData();
        onRetry();
        swipeRefreshLayout.setOnRefreshListener(this::refreshData);

        return view;
    }

    private void onRetry() {
        btn_empty_retry.setOnClickListener(view -> refreshData());
        btn_failed_retry.setOnClickListener(view -> refreshData());
    }

    private void refreshData() {
        categoryList.clear();
        showRefresh(true);
        new Handler().postDelayed(() -> {
            if (tools.isNetworkAvailable()) {
                fetchData();
            } else {
                showRefresh(false);
                lyt_failed.setVisibility(View.VISIBLE);
            }

        }, Constant.DELAY_REFRESH_SHORT);
    }

    private void fetchData() {
        JsonArrayRequest request = new JsonArrayRequest(Constant.URL_CATEGORY_RADIO + Constant.SECURITY_KEY, response -> {
            if (response == null) {
                lyt_failed.setVisibility(View.VISIBLE);
                return;
            }

            List<ItemCategory> items = new Gson().fromJson(response.toString(), new TypeToken<List<ItemCategory>>() {
            }.getType());

            if (items.size() == 0) {
                lyt_empty.setVisibility(View.VISIBLE);
            } else {
                categoryList.clear();
                categoryList.addAll(items);
                mAdapter.notifyDataSetChanged();
                lyt_empty.setVisibility(View.GONE);
                onClick();
            }

            lyt_failed.setVisibility(View.GONE);
            showRefresh(false);

        }, error -> {
            lyt_failed.setVisibility(View.VISIBLE);
            showRefresh(false);
        });

        MyApplication.getInstance().addToRequestQueue(request);
    }

    public void onClick() {

        mAdapter.setOnItemClickListener((v, obj, position) -> {

            new Handler().postDelayed(() -> {
                FragmentManager fragmentManager = null;
                if (FragmentCategory.this.getParentFragment() != null) {
                    fragmentManager = FragmentCategory.this.getParentFragment().getFragmentManager();
                }

                FragmentCategoryDetail f1 = new FragmentCategoryDetail();

                Bundle args = new Bundle();
                args.putString("category_id", obj.getCid());
                args.putString("category_name", obj.getCategory_name());
                f1.setArguments(args);

                assert fragmentManager != null;
                FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                fragmentTransaction.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
                fragmentTransaction.hide(FragmentCategory.this.getParentFragment());
                fragmentTransaction.add(R.id.fragment_container, f1, obj.getCategory_name());
                fragmentTransaction.addToBackStack(obj.getCategory_name());
                fragmentTransaction.commit();
            }, 250);

            showInterstitialAdNetwork();

        });
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            inflater.inflate(R.menu.menu_search_light, menu);
        } else {
            inflater.inflate(R.menu.menu_search_dark, menu);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.search) {
            FragmentManager fm = getParentFragment().getFragmentManager();
            FragmentSearch f1 = new FragmentSearch();

            FragmentTransaction ft = fm.beginTransaction();

            ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
            ft.hide(getParentFragment());
            ft.add(R.id.fragment_container, f1, getString(R.string.menu_search));
            ft.addToBackStack(getString(R.string.menu_search));
            ft.commit();
            return true;
        }

        return false;
    }

    private void loadAdNetwork() {
        if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(ADMOB)) {
            if (!adsPref.getAdMobInterstitialId().equals("0")) {
                adMobInterstitialAd = new InterstitialAd(getActivity());
                adMobInterstitialAd.setAdUnitId(adsPref.getAdMobInterstitialId());
                adMobInterstitialAd.loadAd(Tools.getAdRequest(getActivity()));
                adMobInterstitialAd.setAdListener(new AdListener() {
                    @Override
                    public void onAdClosed() {
                        adMobInterstitialAd.loadAd(Tools.getAdRequest(getActivity()));
                    }
                });
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(FAN)) {
            if (BuildConfig.DEBUG) {
                fanInterstitialAd = new com.facebook.ads.InterstitialAd(getActivity(), "IMG_16_9_APP_INSTALL#" + adsPref.getFanInterstitialUnitId());
            } else {
                fanInterstitialAd = new com.facebook.ads.InterstitialAd(getActivity(), adsPref.getFanInterstitialUnitId());
            }
            com.facebook.ads.InterstitialAdListener adListener = new InterstitialAdListener() {
                @Override
                public void onError(Ad ad, AdError adError) {

                }

                @Override
                public void onAdLoaded(Ad ad) {

                }

                @Override
                public void onAdClicked(Ad ad) {

                }

                @Override
                public void onLoggingImpression(Ad ad) {

                }

                @Override
                public void onInterstitialDisplayed(Ad ad) {

                }

                @Override
                public void onInterstitialDismissed(Ad ad) {
                    fanInterstitialAd.loadAd();
                }
            };

            com.facebook.ads.InterstitialAd.InterstitialLoadAdConfig loadAdConfig = fanInterstitialAd.buildLoadAdConfig().withAdListener(adListener).build();
            fanInterstitialAd.loadAd(loadAdConfig);

        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(STARTAPP)) {
            if (!adsPref.getStartappAppID().equals("0")) {
                startAppAd = new StartAppAd(getActivity());
            }
        }
    }

    private void showInterstitialAdNetwork() {
        if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(ADMOB)) {
            if (!adsPref.getAdMobInterstitialId().equals("0")) {
                if (adMobInterstitialAd != null && adMobInterstitialAd.isLoaded()) {
                    if (counter == adsPref.getInterstitialAdInterval()) {
                        adMobInterstitialAd.show();
                        counter = 1;
                    } else {
                        counter++;
                    }
                }
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(FAN)) {
            if (!adsPref.getFanInterstitialUnitId().equals("0")) {
                if (fanInterstitialAd != null && fanInterstitialAd.isAdLoaded()) {
                    if (counter == adsPref.getInterstitialAdInterval()) {
                        fanInterstitialAd.show();
                        counter = 1;
                    } else {
                        counter++;
                    }
                }
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(STARTAPP)) {
            if (!adsPref.getStartappAppID().equals("0")) {
                if (counter == adsPref.getInterstitialAdInterval()) {
                    startAppAd.showAd();
                    counter = 1;
                } else {
                    counter++;
                }
            }
        }
    }

    private void showRefresh(boolean show) {
        if (show) {
            swipeRefreshLayout.setRefreshing(true);
            lyt_content.setVisibility(View.GONE);
            lyt_failed.setVisibility(View.GONE);
            lyt_empty.setVisibility(View.GONE);
            lyt_shimmer.setVisibility(View.VISIBLE);
            lyt_shimmer.startShimmer();
        } else {
            swipeRefreshLayout.setRefreshing(false);
            lyt_content.setVisibility(View.VISIBLE);
            lyt_shimmer.setVisibility(View.GONE);
            lyt_shimmer.stopShimmer();
        }
    }

    @Override
    public void onDestroy() {
        showRefresh(false);
        lyt_shimmer.stopShimmer();
        super.onDestroy();
    }

}