package com.app.yourradioapp.fragments;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.PopupMenu;
import android.widget.RelativeLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.SearchView;
import androidx.appcompat.widget.Toolbar;
import androidx.coordinatorlayout.widget.CoordinatorLayout;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.android.volley.toolbox.JsonArrayRequest;
import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MainActivity;
import com.app.yourradioapp.activities.MyApplication;
import com.app.yourradioapp.adapters.AdapterCategoryDetail;
import com.app.yourradioapp.models.ItemRadio;
import com.app.yourradioapp.utils.AppBarLayoutBehavior;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.DatabaseHandler;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;
import com.facebook.shimmer.ShimmerFrameLayout;
import com.google.android.material.appbar.AppBarLayout;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import java.util.ArrayList;
import java.util.List;

import static com.app.yourradioapp.utils.Constant.THEME_DARK;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;

public class FragmentCategoryDetail extends Fragment {

    private List<ItemRadio> radioList;
    private AdapterCategoryDetail adapterCategoryDetail;
    RecyclerView recyclerView;
    SearchView searchView;
    SwipeRefreshLayout swipeRefreshLayout = null;
    private CharSequence charSequence = null;
    private DatabaseHandler databaseHandler;
    String cid, category_name;
    private MainActivity mainActivity;
    private Toolbar toolbar;
    View lyt_empty, lyt_failed;
    Button btn_empty_retry, btn_failed_retry;
    RelativeLayout lyt_content;
    ShimmerFrameLayout lyt_shimmer;
    Tools tools;
    private AppBarLayout appBarLayout;
    ThemePref themePref;

    public FragmentCategoryDetail() {

    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        mainActivity = (MainActivity) context;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_category_detail, container, false);

        appBarLayout = view.findViewById(R.id.tab_appbar_layout);
        ((CoordinatorLayout.LayoutParams) appBarLayout.getLayoutParams()).setBehavior(new AppBarLayoutBehavior());

        themePref = new ThemePref(getActivity());

        if (getArguments() != null) {
            cid = getArguments().getString("category_id");
            category_name = getArguments().getString("category_name");
        }

        toolbar = view.findViewById(R.id.toolbar);
        setupToolbar();

        tools = new Tools(getActivity());
        setHasOptionsMenu(true);
        lyt_shimmer = view.findViewById(R.id.shimmer_view_container);
        lyt_content = view.findViewById(R.id.lyt_content);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        swipeRefreshLayout.setColorSchemeResources(R.color.colorPrimary);
        showRefresh(true);

        lyt_empty = view.findViewById(R.id.lyt_empty);
        lyt_failed = view.findViewById(R.id.lyt_no_network);
        btn_empty_retry = view.findViewById(R.id.btn_empty_retry);
        btn_failed_retry = view.findViewById(R.id.btn_failed_retry);

        recyclerView = view.findViewById(R.id.recyclerView);

        radioList = new ArrayList<>();
        adapterCategoryDetail = new AdapterCategoryDetail(getActivity(), radioList);

        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getActivity());
        recyclerView.setLayoutManager(linearLayoutManager);
        recyclerView.setHasFixedSize(true);
        recyclerView.setAdapter(adapterCategoryDetail);

        fetchData();
        onRetry();
        swipeRefreshLayout.setOnRefreshListener(this::refreshData);

        themeColor();

        return view;
    }

    private void onRetry() {
        btn_empty_retry.setOnClickListener(view -> refreshData());
        btn_failed_retry.setOnClickListener(view -> refreshData());
    }

    private void refreshData() {
        radioList.clear();
        showRefresh(true);
        new Handler().postDelayed(() -> {
            if (tools.isNetworkAvailable()) {
                fetchData();
            } else {
                showRefresh(false);
                lyt_failed.setVisibility(View.VISIBLE);
            }

        }, Constant.DELAY_REFRESH_SHORT);
    }

    private void fetchData() {
        JsonArrayRequest request = new JsonArrayRequest(Constant.URL_RADIO_BY_CATEGORY + cid, response -> {
            if (response == null) {
                lyt_failed.setVisibility(View.VISIBLE);
                return;
            }

            List<ItemRadio> items = new Gson().fromJson(response.toString(), new TypeToken<List<ItemRadio>>() {
            }.getType());

            if (items.size() == 0) {
                lyt_empty.setVisibility(View.VISIBLE);
            } else {
                radioList.clear();
                radioList.addAll(items);
                adapterCategoryDetail.notifyDataSetChanged();
                addFavorite();
                lyt_empty.setVisibility(View.GONE);
            }

            lyt_failed.setVisibility(View.GONE);
            showRefresh(false);

        }, error -> {
            lyt_failed.setVisibility(View.VISIBLE);
            showRefresh(false);
        });

        MyApplication.getInstance().addToRequestQueue(request);
    }

    public void addFavorite() {
        adapterCategoryDetail.setOnItemOverflowClickListener((v, obj, position) -> {
            PopupMenu popup = new PopupMenu(getActivity(), v);
            MenuInflater inflater = popup.getMenuInflater();
            inflater.inflate(R.menu.menu_popup, popup.getMenu());
            popup.setOnMenuItemClickListener(item -> {
                switch (item.getItemId()) {
                    case R.id.menu_context_favorite:
                        if (charSequence.equals(getString(R.string.option_set_favorite))) {
                            databaseHandler.AddtoFavorite(new ItemRadio(obj.getRadio_id(), obj.getRadio_name(), obj.getRadio_image(), obj.getRadio_url(), obj.getCategory_name()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_added), Toast.LENGTH_SHORT).show();

                        } else if (charSequence.equals(getString(R.string.option_unset_favorite))) {
                            databaseHandler.RemoveFav(new ItemRadio(obj.getRadio_id()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_removed), Toast.LENGTH_SHORT).show();
                        }
                        return true;

                    case R.id.menu_context_share:

                        String share_title = android.text.Html.fromHtml(obj.getRadio_name()).toString();
                        String share_content = android.text.Html.fromHtml(getResources().getString(R.string.share_content)).toString();
                        Intent sendIntent = new Intent();
                        sendIntent.setAction(Intent.ACTION_SEND);
                        sendIntent.putExtra(Intent.EXTRA_TEXT, share_title + "\n\n" + share_content + "\n\n" + "https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID);
                        sendIntent.setType("text/plain");
                        startActivity(sendIntent);
                        return true;

                    default:
                }
                return false;
            });
            popup.show();

            databaseHandler = new DatabaseHandler(getActivity());
            List<ItemRadio> data = databaseHandler.getFavRow(obj.getRadio_id());
            if (data.size() == 0) {
                popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_set_favorite);
                charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
            } else {
                if (data.get(0).getRadio_id().equals(obj.getRadio_id())) {
                    popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_unset_favorite);
                    charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
                }
            }

        });
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            inflater.inflate(R.menu.menu_search_light, menu);
        } else {
            inflater.inflate(R.menu.menu_search_dark, menu);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.search) {
            Activity act = getActivity();
            if (act instanceof MainActivity) {
                ((MainActivity) act).searchAction();
            }
            return true;
        }

        return false;
    }
    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mainActivity.setupNavigationDrawer(toolbar);
    }

    private void setupToolbar() {
        toolbar.setTitle(category_name);
        mainActivity.setSupportActionBar(toolbar);
    }

    private void showRefresh(boolean show) {
        if (show) {
            swipeRefreshLayout.setRefreshing(true);
            lyt_content.setVisibility(View.GONE);
            lyt_shimmer.setVisibility(View.VISIBLE);
            lyt_shimmer.startShimmer();
        } else {
            swipeRefreshLayout.setRefreshing(false);
            lyt_content.setVisibility(View.VISIBLE);
            lyt_shimmer.setVisibility(View.GONE);
            lyt_shimmer.stopShimmer();
        }
    }

    @Override
    public void onDestroy() {
        showRefresh(false);
        lyt_shimmer.stopShimmer();
        super.onDestroy();
    }

    private void themeColor() {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.grey));

        } else if (themePref.getCurrentTheme() == THEME_DARK) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorBackgroundDark));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorToolbarDark));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.white));
        }
    }

}