package com.app.yourradioapp.fragments;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.PopupMenu;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MainActivity;
import com.app.yourradioapp.adapters.AdapterRadio;
import com.app.yourradioapp.models.ItemRadio;
import com.app.yourradioapp.services.RadioPlayerService;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.DatabaseHandler;
import com.app.yourradioapp.utils.ThemePref;

import java.util.ArrayList;
import java.util.List;

import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;

public class FragmentFavorite extends Fragment {

    private List<ItemRadio> data = new ArrayList<ItemRadio>();
    View root_view;
    AdapterRadio mAdapterFavorite;
    DatabaseHandler databaseHandler;
    RecyclerView recyclerView;
    LinearLayout linearLayout;
    private CharSequence charSequence = null;
    ThemePref themePref;

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        root_view = inflater.inflate(R.layout.fragment_favorite, container, false);
        setHasOptionsMenu(true);
        linearLayout = root_view.findViewById(R.id.lyt_no_favorite);

        recyclerView = root_view.findViewById(R.id.recyclerView);
        RecyclerView.LayoutManager mLayoutManager = new LinearLayoutManager(getActivity());
        recyclerView.setLayoutManager(mLayoutManager);
        recyclerView.setItemAnimator(new DefaultItemAnimator());

        themePref = new ThemePref(getActivity());
        databaseHandler = new DatabaseHandler(getActivity());

        data = databaseHandler.getAllData();
        mAdapterFavorite = new AdapterRadio(getActivity(), (ArrayList<ItemRadio>) data);
        recyclerView.setAdapter(mAdapterFavorite);

        if (data.size() == 0) {
            linearLayout.setVisibility(View.VISIBLE);
        } else {
            linearLayout.setVisibility(View.INVISIBLE);
        }

        return root_view;
    }

    @Override
    public void onResume() {

        data = databaseHandler.getAllData();
        mAdapterFavorite = new AdapterRadio(getActivity(), (ArrayList<ItemRadio>) data);
        recyclerView.setAdapter(mAdapterFavorite);

        mAdapterFavorite.setOnItemClickListener((view, obj, position) -> {
            Constant.item_radio.clear();
            Constant.item_radio.addAll(data);
            Constant.position = position;
            Intent intent = new Intent(getActivity(), RadioPlayerService.class);
            RadioPlayerService.createInstance().initialize(getActivity(), Constant.item_radio.get(position));
            intent.setAction(RadioPlayerService.ACTION_PLAY);
            getActivity().startService(intent);
        });

        addFavorite();

        if (data.size() == 0) {
            linearLayout.setVisibility(View.VISIBLE);
        } else {
            linearLayout.setVisibility(View.INVISIBLE);
        }

        super.onResume();

    }

    public void addFavorite() {
        mAdapterFavorite.setOnItemOverflowClickListener((v, obj, position) -> {
            PopupMenu popup = new PopupMenu(getActivity(), v);
            MenuInflater inflater = popup.getMenuInflater();
            inflater.inflate(R.menu.menu_popup, popup.getMenu());
            popup.setOnMenuItemClickListener(item -> {
                switch (item.getItemId()) {
                    case R.id.menu_context_favorite:
                        if (charSequence.equals(getString(R.string.option_set_favorite))) {
                            databaseHandler.AddtoFavorite(new ItemRadio(obj.getRadio_id(), obj.getRadio_name(), obj.getRadio_image(), obj.getRadio_url(), obj.getCategory_name()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_added), Toast.LENGTH_SHORT).show();
                            updateFav();

                        } else if (charSequence.equals(getString(R.string.option_unset_favorite))) {
                            databaseHandler.RemoveFav(new ItemRadio(obj.getRadio_id()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_removed), Toast.LENGTH_SHORT).show();
                            refreshFragment();
                            updateFav();
                        }
                        return true;

                    case R.id.menu_context_share:

                        String share_title = android.text.Html.fromHtml(obj.getRadio_name()).toString();
                        String share_content = android.text.Html.fromHtml(getResources().getString(R.string.share_content)).toString();
                        Intent sendIntent = new Intent();
                        sendIntent.setAction(Intent.ACTION_SEND);
                        sendIntent.putExtra(Intent.EXTRA_TEXT, share_title + "\n\n" + share_content + "\n\n" + "https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID);
                        sendIntent.setType("text/plain");
                        startActivity(sendIntent);
                        return true;

                    default:
                }
                return false;
            });
            popup.show();

            databaseHandler = new DatabaseHandler(getActivity());
            List<ItemRadio> data = databaseHandler.getFavRow(obj.getRadio_id());
            if (data.size() == 0) {
                popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_set_favorite);
                charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
            } else {
                if (data.get(0).getRadio_id().equals(obj.getRadio_id())) {
                    popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_unset_favorite);
                    charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
                }
            }

        });
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, MenuInflater inflater) {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            inflater.inflate(R.menu.menu_search_light, menu);
        } else {
            inflater.inflate(R.menu.menu_search_dark, menu);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.search) {
            FragmentManager fm = getParentFragment().getFragmentManager();
            FragmentSearch f1 = new FragmentSearch();

            FragmentTransaction ft = fm.beginTransaction();

            ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
            ft.hide(getParentFragment());
            ft.add(R.id.fragment_container, f1, getString(R.string.menu_search));
            ft.addToBackStack(getString(R.string.menu_search));
            ft.commit();
            return true;
        }

        return false;
    }

    public void refreshFragment() {
        assert getFragmentManager() != null;
        FragmentTransaction fragmentTransaction = getFragmentManager().beginTransaction();
        fragmentTransaction.detach(this).attach(this).commit();
    }

    private void updateFav() {
        if (getActivity() != null)
            ((MainActivity) getActivity()).changeFav(Constant.item_radio.get(0));
    }

}
