package com.app.yourradioapp.fragments;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.PopupMenu;
import android.widget.RelativeLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MainActivity;
import com.app.yourradioapp.adapters.AdapterRadio;
import com.app.yourradioapp.models.ItemRadio;
import com.app.yourradioapp.services.RadioPlayerService;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.DatabaseHandler;
import com.app.yourradioapp.utils.RadioTask;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;
import com.facebook.shimmer.ShimmerFrameLayout;

import java.util.ArrayList;
import java.util.List;

import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;

public class FragmentRadio extends Fragment {

    ArrayList<ItemRadio> arrayList_radio_latest;
    RecyclerView recyclerView;
    RadioTask radioTask;
    @SuppressLint("StaticFieldLeak")
    public static AdapterRadio adapterRadio;
    private CharSequence charSequence = null;
    private DatabaseHandler databaseHandler;
    View lyt_empty, lyt_failed;
    Button btn_empty_retry, btn_failed_retry;
    private SwipeRefreshLayout swipeRefreshLayout;
    RelativeLayout lyt_parent;
    RelativeLayout lyt_content;
    ShimmerFrameLayout lyt_shimmer;
    Tools tools;
    ThemePref themePref;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_radio, container, false);
        lyt_parent = view.findViewById(R.id.lyt_parent);
        setHasOptionsMenu(true);
        themePref = new ThemePref(getActivity());
        tools = new Tools(getActivity());

        arrayList_radio_latest = new ArrayList<>();
        lyt_shimmer = view.findViewById(R.id.shimmer_view_container);
        lyt_content = view.findViewById(R.id.lyt_content);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        swipeRefreshLayout.setColorSchemeResources(R.color.colorPrimary);

        lyt_empty = view.findViewById(R.id.lyt_empty);
        lyt_failed = view.findViewById(R.id.lyt_no_network);
        btn_empty_retry = view.findViewById(R.id.btn_empty_retry);
        btn_failed_retry = view.findViewById(R.id.btn_failed_retry);

        recyclerView = view.findViewById(R.id.recyclerView);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getActivity());
        recyclerView.setLayoutManager(linearLayoutManager);
        recyclerView.setHasFixedSize(true);
        adapterRadio = new AdapterRadio(getActivity(), arrayList_radio_latest);
        recyclerView.setAdapter(adapterRadio);

        if (tools.isNetworkAvailable()) {
            RadioTask();
        } else {
            lyt_failed.setVisibility(View.VISIBLE);
            showRefresh(false);
        }

        onRetry();

        swipeRefreshLayout.setOnRefreshListener(this::refreshData);

        return view;
    }

    private void onRetry() {
        btn_empty_retry.setOnClickListener(v -> refreshData());
        btn_failed_retry.setOnClickListener(v -> refreshData());
    }

    private void refreshData() {
        showRefresh(true);
        arrayList_radio_latest.clear();
        new Handler().postDelayed(() -> {
            if (tools.isNetworkAvailable()) {
                RadioTask();
                adapterRadio.notifyDataSetChanged();
                adapterRadio.notifyItemRangeInserted(0, arrayList_radio_latest.size());
            } else {
                lyt_failed.setVisibility(View.VISIBLE);
                showRefresh(false);
            }

        }, Constant.DELAY_REFRESH_SHORT);
    }

    private void RadioTask() {
        radioTask = new RadioTask(new RadioTask.RadioListListener() {
            @Override
            public void onStart() {
                if (getActivity() != null) {
                    arrayList_radio_latest.clear();
                    showRefresh(true);
                }
            }

            @Override
            public void onEnd(String success, ArrayList<ItemRadio> arrayList) {
                if (getActivity() != null) {
                    arrayList_radio_latest.addAll(arrayList);
                    showRefresh(false);
                    if (arrayList.size() > 0) {
                        adapterRadio = new AdapterRadio(getActivity(), arrayList_radio_latest);
                        recyclerView.setAdapter(adapterRadio);
                        if (Constant.item_radio.size() == 0) {
                            Constant.item_radio.addAll(arrayList_radio_latest);
                            ((MainActivity) getActivity()).changeText(Constant.item_radio.get(0));
                        }

                        adapterRadio.setOnItemClickListener((view, obj, position) -> {
                            Constant.item_radio.clear();
                            Constant.item_radio.addAll(arrayList);
                            Constant.position = position;
                            Intent intent = new Intent(getActivity(), RadioPlayerService.class);
                            RadioPlayerService.createInstance().initialize(getActivity(), Constant.item_radio.get(position));
                            intent.setAction(RadioPlayerService.ACTION_PLAY);
                            getActivity().startService(intent);
                        });

                        addFavorite();
                        lyt_empty.setVisibility(View.GONE);
                    } else {
                        lyt_empty.setVisibility(View.VISIBLE);
                    }
                }
            }
        });
        radioTask.execute(Constant.URL_RECENT_RADIO + Constant.SECURITY_KEY);
        lyt_failed.setVisibility(View.GONE);
    }

    public void addFavorite() {
        adapterRadio.setOnItemOverflowClickListener((v, obj, position) -> {
            PopupMenu popup = new PopupMenu(getActivity(), v);
            MenuInflater inflater = popup.getMenuInflater();
            inflater.inflate(R.menu.menu_popup, popup.getMenu());
            popup.setOnMenuItemClickListener(item -> {
                switch (item.getItemId()) {
                    case R.id.menu_context_favorite:
                        if (charSequence.equals(getString(R.string.option_set_favorite))) {
                            databaseHandler.AddtoFavorite(new ItemRadio(obj.getRadio_id(), obj.getRadio_name(), obj.getRadio_image(), obj.getRadio_url(), obj.getCategory_name()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_added), Toast.LENGTH_SHORT).show();
                            updateFav();

                        } else if (charSequence.equals(getString(R.string.option_unset_favorite))) {
                            databaseHandler.RemoveFav(new ItemRadio(obj.getRadio_id()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_removed), Toast.LENGTH_SHORT).show();
                            updateFav();
                        }
                        return true;

                    case R.id.menu_context_share:

                        String share_title = android.text.Html.fromHtml(obj.getRadio_name()).toString();
                        String share_content = android.text.Html.fromHtml(getResources().getString(R.string.share_content)).toString();
                        Intent sendIntent = new Intent();
                        sendIntent.setAction(Intent.ACTION_SEND);
                        sendIntent.putExtra(Intent.EXTRA_TEXT, share_title + "\n\n" + share_content + "\n\n" + "https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID);
                        sendIntent.setType("text/plain");
                        startActivity(sendIntent);
                        return true;

                    default:
                }
                return false;
            });
            popup.show();

            databaseHandler = new DatabaseHandler(getActivity());
            List<ItemRadio> data = databaseHandler.getFavRow(obj.getRadio_id());
            if (data.size() == 0) {
                popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_set_favorite);
                charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
            } else {
                if (data.get(0).getRadio_id().equals(obj.getRadio_id())) {
                    popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_unset_favorite);
                    charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
                }
            }

        });
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, @NonNull MenuInflater inflater) {
        menu.clear();
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            inflater.inflate(R.menu.menu_search_light, menu);
        } else {
            inflater.inflate(R.menu.menu_search_dark, menu);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.search) {
            FragmentManager fm = null;
            if (getParentFragment() != null) {
                fm = getParentFragment().getFragmentManager();
            }
            FragmentSearch f1 = new FragmentSearch();

            assert fm != null;
            FragmentTransaction ft = fm.beginTransaction();
            ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
            ft.hide(getParentFragment());
            ft.add(R.id.fragment_container, f1, getString(R.string.menu_search));
            ft.addToBackStack(getString(R.string.menu_search));
            ft.commit();
            return true;
        }

        if (item.getItemId() == R.id.search) {
            if (getActivity() != null)
                ((MainActivity) getActivity()).openTimeDialog();
        }

        return false;
    }

    private void showRefresh(boolean show) {
        if (show) {
            swipeRefreshLayout.setRefreshing(true);
            lyt_content.setVisibility(View.GONE);
            lyt_failed.setVisibility(View.GONE);
            lyt_empty.setVisibility(View.GONE);
            lyt_shimmer.setVisibility(View.VISIBLE);
            lyt_shimmer.startShimmer();
        } else {
            swipeRefreshLayout.setRefreshing(false);
            lyt_content.setVisibility(View.VISIBLE);
            lyt_shimmer.setVisibility(View.GONE);
            lyt_shimmer.stopShimmer();
        }
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onDestroy() {
        showRefresh(false);
        lyt_shimmer.stopShimmer();
        super.onDestroy();
    }

    private void updateFav() {
        if (getActivity() != null)
            ((MainActivity) getActivity()).changeFav(Constant.item_radio.get(0));
    }

}
