package com.app.yourradioapp.fragments;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.MenuInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.PopupMenu;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.Toolbar;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MainActivity;
import com.app.yourradioapp.adapters.AdapterRadio;
import com.app.yourradioapp.models.ItemRadio;
import com.app.yourradioapp.services.parser.JSONParser;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.DatabaseHandler;
import com.app.yourradioapp.utils.SharedPref;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import static com.app.yourradioapp.utils.Constant.THEME_DARK;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;

public class FragmentSearch extends Fragment {

    RecyclerView recyclerView;
    AdapterRadio adapterRadio;
    ArrayList<ItemRadio> itemRadios;
    ProgressBar progressBar;
    Button btn_empty_retry, btn_failed_retry;
    View lyt_empty, lyt_failed;
    SharedPref sharedPref;
    private EditText et_search;
    private ImageButton bt_clear;
    private MainActivity mainActivity;
    private Toolbar toolbar;
    private CharSequence charSequence = null;
    private DatabaseHandler databaseHandler;
    Tools tools;
    ThemePref themePref;

    public FragmentSearch() {

    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        mainActivity = (MainActivity) context;
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_search, container, false);

        themePref = new ThemePref(getActivity());

        toolbar = rootView.findViewById(R.id.toolbar);
        setupToolbar();

        et_search = rootView.findViewById(R.id.et_search);
        et_search.requestFocus();
        et_search.setFocusable(true);

        bt_clear = rootView.findViewById(R.id.bt_clear);
        bt_clear.setVisibility(View.GONE);

        et_search.addTextChangedListener(textWatcher);
        bt_clear.setOnClickListener(view -> et_search.setText(""));

        et_search.setOnEditorActionListener((v, actionId, event) -> {
            if (actionId == EditorInfo.IME_ACTION_SEARCH) {
                hideKeyboard(getActivity());
                searchAction();
                return true;
            }
            return false;
        });

        if (getActivity() != null)
        sharedPref = new SharedPref(getActivity());
        tools = new Tools(getActivity());

        itemRadios = new ArrayList<>();
        progressBar = rootView.findViewById(R.id.progressBar);

        lyt_empty = rootView.findViewById(R.id.lyt_empty);
        lyt_failed = rootView.findViewById(R.id.lyt_no_network);
        btn_empty_retry = rootView.findViewById(R.id.btn_empty_retry);
        btn_failed_retry = rootView.findViewById(R.id.btn_failed_retry);

        recyclerView = rootView.findViewById(R.id.recyclerView);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getActivity());
        recyclerView.setLayoutManager(linearLayoutManager);
        recyclerView.setHasFixedSize(true);

        btn_empty_retry.setOnClickListener(v -> searchAction());
        btn_failed_retry.setOnClickListener(v -> searchAction());

        themeColor();

        showKeyboard(getActivity());

        return rootView;
    }

    private void searchAction() {
        if (tools.isNetworkAvailable()) {
            final String query = et_search.getText().toString().trim();
            if (!query.equals("")) {
                new Handler().postDelayed(() -> new RadioTask().execute(Constant.URL_RADIO_SEARCH + query + Constant.SECURITY_KEY), 250);
            } else {
                Toast.makeText(getActivity(), getResources().getString(R.string.msg_search_input), Toast.LENGTH_SHORT).show();
            }
        } else {
            lyt_failed.setVisibility(View.VISIBLE);
        }
    }

    @SuppressWarnings("deprecation")
    @SuppressLint("StaticFieldLeak")
    private class RadioTask extends AsyncTask<String, Void, String> {
        @Override
        protected void onPreExecute() {
            itemRadios.clear();
            lyt_empty.setVisibility(View.GONE);
            progressBar.setVisibility(View.VISIBLE);
            recyclerView.setVisibility(View.GONE);
            super.onPreExecute();
        }

        @Override
        protected String doInBackground(String... params) {
            return JSONParser.okHttpGET(params[0]);
        }

        @Override
        protected void onPostExecute(String result) {
            super.onPostExecute(result);
            if (getActivity() != null) {
                if (null == result || result.length() == 0) {
                    lyt_empty.setVisibility(View.VISIBLE);
                } else {
                    try {
                        JSONObject mainJson = new JSONObject(result);
                        JSONArray jsonArray = mainJson.getJSONArray(Constant.JSON_ARRAY_NAME);
                        JSONObject objJson;

                        for (int i = 0; i < jsonArray.length(); i++) {
                            objJson = jsonArray.getJSONObject(i);

                            String radio_id = objJson.getString(Constant.RECENT_RADIO_ID);
                            String radio_name = objJson.getString(Constant.RECENT_RADIO_NAME);
                            String radio_image = objJson.getString(Constant.RECENT_RADIO_IMAGE);
                            String radio_url = objJson.getString(Constant.RECENT_RADIO_URL);
                            String category_name = objJson.getString(Constant.RECENT_CATEGORY_NAME);

                            ItemRadio objItem = new ItemRadio(radio_id, radio_name, radio_image, radio_url, category_name);
                            itemRadios.add(objItem);
                        }
                        lyt_empty.setVisibility(View.VISIBLE);
                    } catch (Exception e) {
                        lyt_failed.setVisibility(View.VISIBLE);
                        e.printStackTrace();
                    }
                    setAdapterToRecyclerView();
                    addFavorite();
                }
            }
        }
    }

    public void setAdapterToRecyclerView() {
        adapterRadio = new AdapterRadio(getActivity(), itemRadios);
        recyclerView.setAdapter(adapterRadio);
        setEmpty();

        adapterRadio.setOnItemClickListener((view, obj, position) -> {
            Constant.item_radio.clear();
            Constant.item_radio.addAll(itemRadios);
            if (getActivity() != null)
            ((MainActivity) getActivity()).clickPlay(position, true);
        });
    }

    public void addFavorite() {
        adapterRadio.setOnItemOverflowClickListener((v, obj, position) -> {
            PopupMenu popup = new PopupMenu(getActivity(), v);
            MenuInflater inflater = popup.getMenuInflater();
            inflater.inflate(R.menu.menu_popup, popup.getMenu());
            popup.setOnMenuItemClickListener(item -> {
                switch (item.getItemId()) {
                    case R.id.menu_context_favorite:
                        if (charSequence.equals(getString(R.string.option_set_favorite))) {
                            databaseHandler.AddtoFavorite(new ItemRadio(obj.getRadio_id(), obj.getRadio_name(), obj.getRadio_image(), obj.getRadio_url(), obj.getCategory_name()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_added), Toast.LENGTH_SHORT).show();

                        } else if (charSequence.equals(getString(R.string.option_unset_favorite))) {
                            databaseHandler.RemoveFav(new ItemRadio(obj.getRadio_id()));
                            Toast.makeText(getActivity(), getString(R.string.favorite_removed), Toast.LENGTH_SHORT).show();
                        }
                        return true;

                    case R.id.menu_context_share:
                        String share_title = android.text.Html.fromHtml(obj.getRadio_name()).toString();
                        String share_content = android.text.Html.fromHtml(getResources().getString(R.string.share_content)).toString();
                        Intent sendIntent = new Intent();
                        sendIntent.setAction(Intent.ACTION_SEND);
                        sendIntent.putExtra(Intent.EXTRA_TEXT, share_title + "\n\n" + share_content + "\n\n" + "https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID);
                        sendIntent.setType("text/plain");
                        startActivity(sendIntent);
                        return true;

                    default:
                }
                return false;
            });
            popup.show();

            databaseHandler = new DatabaseHandler(getActivity());
            List<ItemRadio> data = databaseHandler.getFavRow(obj.getRadio_id());
            if (data.size() == 0) {
                popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_set_favorite);
                charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
            } else {
                if (data.get(0).getRadio_id().equals(obj.getRadio_id())) {
                    popup.getMenu().findItem(R.id.menu_context_favorite).setTitle(R.string.option_unset_favorite);
                    charSequence = popup.getMenu().findItem(R.id.menu_context_favorite).getTitle();
                }
            }

        });
    }

    public void setEmpty() {
        progressBar.setVisibility(View.GONE);
        if (itemRadios.size() > 0) {
            recyclerView.setVisibility(View.VISIBLE);
            lyt_empty.setVisibility(View.GONE);
        } else {
            recyclerView.setVisibility(View.GONE);
            lyt_empty.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mainActivity.setupNavigationDrawer(toolbar);
    }

    private void setupToolbar() {
        toolbar.setTitle(getResources().getString(R.string.menu_search));
        mainActivity.setSupportActionBar(toolbar);
    }

    TextWatcher textWatcher = new TextWatcher() {
        @Override
        public void onTextChanged(CharSequence c, int i, int i1, int i2) {
            if (c.toString().trim().length() == 0) {
                bt_clear.setVisibility(View.GONE);
            } else {
                bt_clear.setVisibility(View.VISIBLE);
            }
        }

        @Override
        public void beforeTextChanged(CharSequence c, int i, int i1, int i2) {
        }

        @Override
        public void afterTextChanged(Editable editable) {
        }
    };

    public static void hideKeyboard(Context context) {
        try {
            ((Activity) context).getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
            if ((((Activity) context).getCurrentFocus() != null) && (((Activity) context).getCurrentFocus().getWindowToken() != null)) {
                ((InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE)).hideSoftInputFromWindow(((Activity) context).getCurrentFocus().getWindowToken(), 0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void showKeyboard(Context context) {
        ((InputMethodManager) (context).getSystemService(Context.INPUT_METHOD_SERVICE)).toggleSoftInput(InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY);
    }

    public void themeColor() {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            et_search.setTextColor(ContextCompat.getColor(getActivity(), R.color.grey));
            et_search.setHintTextColor(ContextCompat.getColor(getActivity(), R.color.grey));
            bt_clear.setColorFilter(ContextCompat.getColor(getActivity(), R.color.grey));
        } else if (themePref.getCurrentTheme() == THEME_DARK) {
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorToolbarDark));
            et_search.setTextColor(ContextCompat.getColor(getActivity(), R.color.white));
            et_search.setHintTextColor(ContextCompat.getColor(getActivity(), R.color.white));
            bt_clear.setColorFilter(ContextCompat.getColor(getActivity(), R.color.white));
        }
    }

}