package com.app.yourradioapp.fragments;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.text.Html;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.Toolbar;
import androidx.coordinatorlayout.widget.CoordinatorLayout;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;

import com.android.volley.Request;
import com.android.volley.toolbox.JsonObjectRequest;
import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MainActivity;
import com.app.yourradioapp.activities.MyApplication;
import com.app.yourradioapp.utils.AppBarLayoutBehavior;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;
import com.google.android.material.appbar.AppBarLayout;

import org.json.JSONException;

import static com.app.yourradioapp.utils.Constant.THEME_DARK;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;
import static com.app.yourradioapp.utils.Constant.THEME_PRIMARY;

public class FragmentSettings extends Fragment {

    private MainActivity mainActivity;
    private Toolbar toolbar;
    Tools tools;
    private AppBarLayout appBarLayout;
    ThemePref themePref;
    private String single_choice_selected;

    public FragmentSettings() {

    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        mainActivity = (MainActivity) context;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_settings, container, false);

        themePref = new ThemePref(getActivity());

        appBarLayout = view.findViewById(R.id.tab_appbar_layout);
        ((CoordinatorLayout.LayoutParams) appBarLayout.getLayoutParams()).setBehavior(new AppBarLayoutBehavior());

        setHasOptionsMenu(false);

        toolbar = view.findViewById(R.id.toolbar);
        setupToolbar();

        tools = new Tools(getActivity());

        themeColor();
        changeTheme(view);
        getPrivacyPolicy(view);
        rateApps(view);
        moreApps(view);

        return view;
    }

    private void changeTheme(View view) {
        final TextView txt_current_theme = view.findViewById(R.id.txt_current_theme);
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            txt_current_theme.setText(getResources().getString(R.string.theme_light));
        } else if (themePref.getCurrentTheme() == THEME_DARK) {
            txt_current_theme.setText(getResources().getString(R.string.theme_dark));
        } else if (themePref.getCurrentTheme() == THEME_PRIMARY) {
            txt_current_theme.setText(getResources().getString(R.string.theme_primary));
        }

        view.findViewById(R.id.btn_switch_theme).setOnClickListener(v -> {

            String[] items = getResources().getStringArray(R.array.dialog_set_theme);
            single_choice_selected = items[themePref.getCurrentTheme()];
            int itemSelected = themePref.getCurrentTheme();
            new AlertDialog.Builder(getActivity())
                    .setTitle(R.string.dialog_title_theme)
                    .setSingleChoiceItems(items, itemSelected, (dialogInterface, i) -> single_choice_selected = items[i])
                    .setPositiveButton(R.string.option_ok, (dialogInterface, i) -> {
                        if (single_choice_selected.equals(getResources().getString(R.string.theme_light))) {
                            themePref.updateTheme(THEME_LIGHT);
                        } else if (single_choice_selected.equals(getResources().getString(R.string.theme_dark))) {
                            themePref.updateTheme(THEME_DARK);
                        } else if (single_choice_selected.equals(getResources().getString(R.string.theme_primary))) {
                            themePref.updateTheme(THEME_PRIMARY);
                        }
                        getActivity().finish();
                        startActivity(getActivity().getIntent());
                        getActivity().overridePendingTransition(0, 0);

                    })
                    .setNegativeButton(R.string.cancel, null)
                    .show();
        });

    }

    public void getPrivacyPolicy(View view) {
        JsonObjectRequest jsonObjReq = new JsonObjectRequest(Request.Method.GET, Constant.URL_PRIVACY_POLICY, null, response -> {
            try {
                final String result = response.getString("privacy_policy");
                view.findViewById(R.id.btn_privacy_policy).setOnClickListener(v -> {
                    if (getActivity() != null) {
                        AlertDialog.Builder dialog = new AlertDialog.Builder(getActivity());
                        dialog.setTitle(getString(R.string.title_about_privacy));
                        dialog.setMessage(Html.fromHtml(result));
                        dialog.setPositiveButton(getString(R.string.option_ok), null);
                        dialog.show();
                    }
                });
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }, error -> Log.d("ERROR", error.getMessage() + "")
        );
        MyApplication.getInstance().addToRequestQueue(jsonObjReq);
    }

    public void rateApps(View view) {
        view.findViewById(R.id.btn_rate).setOnClickListener(v -> {
            final String package_name = BuildConfig.APPLICATION_ID;
            try {
                startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=" + package_name)));
            } catch (android.content.ActivityNotFoundException anfe) {
                startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("http://play.google.com/store/apps/details?id=" + package_name)));
            }
        });
    }

    public void moreApps(View view) {
        view.findViewById(R.id.btn_more).setOnClickListener(v -> startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(getString(R.string.play_more_apps)))));
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.menu, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mainActivity.setupNavigationDrawer(toolbar);
    }

    private void setupToolbar() {
        toolbar.setTitle(R.string.toolbar_title_settings);
        mainActivity.setSupportActionBar(toolbar);
    }

    private void themeColor() {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.grey));

        } else if (themePref.getCurrentTheme() == THEME_DARK) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorBackgroundDark));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorToolbarDark));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.white));
        }
    }

}