package com.app.yourradioapp.fragments;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RelativeLayout;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.Toolbar;
import androidx.coordinatorlayout.widget.CoordinatorLayout;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.android.volley.toolbox.JsonArrayRequest;
import com.app.yourradioapp.BuildConfig;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MainActivity;
import com.app.yourradioapp.activities.MyApplication;
import com.app.yourradioapp.adapters.AdapterSocial;
import com.app.yourradioapp.models.ItemSocial;
import com.app.yourradioapp.utils.AdsPref;
import com.app.yourradioapp.utils.AppBarLayoutBehavior;
import com.app.yourradioapp.utils.Constant;
import com.app.yourradioapp.utils.ThemePref;
import com.app.yourradioapp.utils.Tools;
import com.facebook.ads.Ad;
import com.facebook.ads.AdError;
import com.facebook.ads.InterstitialAdListener;
import com.facebook.shimmer.ShimmerFrameLayout;
import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.InterstitialAd;
import com.google.android.material.appbar.AppBarLayout;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.startapp.sdk.adsbase.StartAppAd;

import java.util.ArrayList;
import java.util.List;

import static com.app.yourradioapp.utils.Constant.ADMOB;
import static com.app.yourradioapp.utils.Constant.AD_STATUS_ON;
import static com.app.yourradioapp.utils.Constant.FAN;
import static com.app.yourradioapp.utils.Constant.STARTAPP;
import static com.app.yourradioapp.utils.Constant.THEME_DARK;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;

public class FragmentSocial extends Fragment {

    private MainActivity mainActivity;
    private Toolbar toolbar;
    private List<ItemSocial> socialList;
    private AdapterSocial mAdapter;
    SwipeRefreshLayout swipeRefreshLayout = null;
    RecyclerView recyclerView;
    View lyt_empty, lyt_failed;
    Button btn_empty_retry, btn_failed_retry;
    RelativeLayout lyt_parent;
    RelativeLayout lyt_content;
    ShimmerFrameLayout lyt_shimmer;
    private InterstitialAd adMobInterstitialAd;
    private com.facebook.ads.InterstitialAd fanInterstitialAd;
    private StartAppAd startAppAd;
    private AdsPref adsPref;
    int counter = 1;
    Tools tools;
    private AppBarLayout appBarLayout;
    ThemePref themePref;

    public FragmentSocial() {

    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        mainActivity = (MainActivity) context;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_social, container, false);
        lyt_parent = view.findViewById(R.id.lyt_parent);

        themePref = new ThemePref(getActivity());

        appBarLayout = view.findViewById(R.id.tab_appbar_layout);
        ((CoordinatorLayout.LayoutParams) appBarLayout.getLayoutParams()).setBehavior(new AppBarLayoutBehavior());

        setHasOptionsMenu(true);

        toolbar = view.findViewById(R.id.toolbar);
        setupToolbar();

        tools = new Tools(getActivity());
        lyt_shimmer = view.findViewById(R.id.shimmer_view_container);
        lyt_content = view.findViewById(R.id.lyt_content);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        swipeRefreshLayout.setColorSchemeResources(R.color.colorPrimary);

        adsPref = new AdsPref(getActivity());
        loadAdNetwork();

        lyt_empty = view.findViewById(R.id.lyt_empty);
        lyt_failed = view.findViewById(R.id.lyt_no_network);
        btn_empty_retry = view.findViewById(R.id.btn_empty_retry);
        btn_failed_retry = view.findViewById(R.id.btn_failed_retry);

        recyclerView = view.findViewById(R.id.recyclerView);
        socialList = new ArrayList<>();
        mAdapter = new AdapterSocial(getActivity(), socialList);

        recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
        recyclerView.setHasFixedSize(true);
        recyclerView.setAdapter(mAdapter);

        fetchData();
        onRetry();
        swipeRefreshLayout.setOnRefreshListener(this::refreshData);

        themeColor();

        return view;
    }

    private void onRetry() {
        btn_empty_retry.setOnClickListener(view -> refreshData());

        btn_failed_retry.setOnClickListener(view -> refreshData());
    }

    private void refreshData() {
        socialList.clear();
        showRefresh(true);
        new Handler().postDelayed(() -> {
            if (tools.isNetworkAvailable()) {
                fetchData();
            } else {
                showRefresh(false);
                lyt_failed.setVisibility(View.VISIBLE);
            }

        }, 1500);
    }

    private void fetchData() {
        JsonArrayRequest request = new JsonArrayRequest(Constant.URL_SOCIAL, response -> {
            if (response == null) {
                lyt_failed.setVisibility(View.VISIBLE);
                return;
            }

            List<ItemSocial> items = new Gson().fromJson(response.toString(), new TypeToken<List<ItemSocial>>() {
            }.getType());

            if (items.size() == 0) {
                lyt_empty.setVisibility(View.VISIBLE);
            } else {
                socialList.clear();
                socialList.addAll(items);
                mAdapter.notifyDataSetChanged();
                lyt_empty.setVisibility(View.GONE);
                onClick();
            }

            lyt_failed.setVisibility(View.GONE);
            showRefresh(false);

        }, error -> {
            lyt_failed.setVisibility(View.VISIBLE);
            showRefresh(false);
        });

        MyApplication.getInstance().addToRequestQueue(request);
    }

    public void onClick() {

        mAdapter.setOnItemClickListener((v, obj, position) -> {
            Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(obj.getUrl()));
            startActivity(intent);

            showInterstitialAdNetwork();

        });
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.menu, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    private void loadAdNetwork() {
        if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(ADMOB)) {
            if (!adsPref.getAdMobInterstitialId().equals("0")) {
                adMobInterstitialAd = new InterstitialAd(getActivity());
                adMobInterstitialAd.setAdUnitId(adsPref.getAdMobInterstitialId());
                adMobInterstitialAd.loadAd(Tools.getAdRequest(getActivity()));
                adMobInterstitialAd.setAdListener(new AdListener() {
                    @Override
                    public void onAdClosed() {
                        adMobInterstitialAd.loadAd(Tools.getAdRequest(getActivity()));
                    }
                });
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(FAN)) {
            if (BuildConfig.DEBUG) {
                fanInterstitialAd = new com.facebook.ads.InterstitialAd(getActivity(), "IMG_16_9_APP_INSTALL#" + adsPref.getFanInterstitialUnitId());
            } else {
                fanInterstitialAd = new com.facebook.ads.InterstitialAd(getActivity(), adsPref.getFanInterstitialUnitId());
            }
            InterstitialAdListener adListener = new InterstitialAdListener() {
                @Override
                public void onError(Ad ad, AdError adError) {

                }

                @Override
                public void onAdLoaded(Ad ad) {

                }

                @Override
                public void onAdClicked(Ad ad) {

                }

                @Override
                public void onLoggingImpression(Ad ad) {

                }

                @Override
                public void onInterstitialDisplayed(Ad ad) {

                }

                @Override
                public void onInterstitialDismissed(Ad ad) {
                    fanInterstitialAd.loadAd();
                }
            };

            com.facebook.ads.InterstitialAd.InterstitialLoadAdConfig loadAdConfig = fanInterstitialAd.buildLoadAdConfig().withAdListener(adListener).build();
            fanInterstitialAd.loadAd(loadAdConfig);

        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(STARTAPP)) {
            if (!adsPref.getStartappAppID().equals("0")) {
                startAppAd = new StartAppAd(getActivity());
            }
        }
    }

    private void showInterstitialAdNetwork() {
        if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(ADMOB)) {
            if (!adsPref.getAdMobInterstitialId().equals("0")) {
                if (adMobInterstitialAd != null && adMobInterstitialAd.isLoaded()) {
                    if (counter == adsPref.getInterstitialAdInterval()) {
                        adMobInterstitialAd.show();
                        counter = 1;
                    } else {
                        counter++;
                    }
                }
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(FAN)) {
            if (!adsPref.getFanInterstitialUnitId().equals("0")) {
                if (fanInterstitialAd != null && fanInterstitialAd.isAdLoaded()) {
                    if (counter == adsPref.getInterstitialAdInterval()) {
                        fanInterstitialAd.show();
                        counter = 1;
                    } else {
                        counter++;
                    }
                }
            }
        } else if (adsPref.getAdStatus().equals(AD_STATUS_ON) && adsPref.getAdType().equals(STARTAPP)) {
            if (!adsPref.getStartappAppID().equals("0")) {
                if (counter == adsPref.getInterstitialAdInterval()) {
                    startAppAd.showAd();
                    counter = 1;
                } else {
                    counter++;
                }
            }
        }
    }

    private void showRefresh(boolean show) {
        if (show) {
            swipeRefreshLayout.setRefreshing(true);
            lyt_content.setVisibility(View.GONE);
            lyt_failed.setVisibility(View.GONE);
            lyt_empty.setVisibility(View.GONE);
            lyt_shimmer.setVisibility(View.VISIBLE);
            lyt_shimmer.startShimmer();
        } else {
            swipeRefreshLayout.setRefreshing(false);
            lyt_content.setVisibility(View.VISIBLE);
            lyt_shimmer.setVisibility(View.GONE);
            lyt_shimmer.stopShimmer();
        }
    }

    @Override
    public void onDestroy() {
        showRefresh(false);
        lyt_shimmer.stopShimmer();
        super.onDestroy();
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mainActivity.setupNavigationDrawer(toolbar);
    }

    private void setupToolbar() {
        toolbar.setTitle(R.string.toolbar_title_social);
        mainActivity.setSupportActionBar(toolbar);
    }

    private void themeColor() {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.grey));

        } else if (themePref.getCurrentTheme() == THEME_DARK) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorBackgroundDark));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorToolbarDark));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.white));
        }
    }

}