package com.app.yourradioapp.fragments;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;
import androidx.coordinatorlayout.widget.CoordinatorLayout;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.app.yourradioapp.Config;
import com.app.yourradioapp.R;
import com.app.yourradioapp.activities.MainActivity;
import com.app.yourradioapp.utils.AppBarLayoutBehavior;
import com.app.yourradioapp.utils.ThemePref;
import com.duolingo.open.rtlviewpager.RtlViewPager;
import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.tabs.TabLayout;

import static com.app.yourradioapp.utils.Constant.THEME_DARK;
import static com.app.yourradioapp.utils.Constant.THEME_LIGHT;
import static com.app.yourradioapp.utils.Constant.THEME_PRIMARY;

public class FragmentTabLayout extends Fragment {

    private MainActivity mainActivity;
    private Toolbar toolbar;
    public TabLayout tabLayout;
    public ViewPager viewPager;
    public RtlViewPager viewPagerRTL;
    private AppBarLayout appBarLayout;
    public int tab_count = 3;
    View view;
    private int[] tabIcons = {
            R.drawable.ic_tab_recent,
            R.drawable.ic_tab_category,
            R.drawable.ic_tab_favorite
    };
    ThemePref themePref;

    public FragmentTabLayout() {

    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        mainActivity = (MainActivity) context;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        if (Config.ENABLE_RTL_MODE) {
            view = inflater.inflate(R.layout.fragment_tab_layout_rtl, container, false);
        } else {
            view = inflater.inflate(R.layout.fragment_tab_layout, container, false);
        }

        appBarLayout = view.findViewById(R.id.tab_appbar_layout);
        ((CoordinatorLayout.LayoutParams) appBarLayout.getLayoutParams()).setBehavior(new AppBarLayoutBehavior());

        themePref = new ThemePref(getActivity());

        toolbar = view.findViewById(R.id.toolbar);
        setupToolbar();

        tabLayout = view.findViewById(R.id.tabs);

        if (Config.ENABLE_RTL_MODE) {
            initViewPagerRTL();
        } else {
            initViewPager();
        }

        themeColor();

        return view;

    }

    private void initViewPager() {
        viewPager = view.findViewById(R.id.viewpager);
        viewPager.setOffscreenPageLimit(tab_count);
        viewPager.setAdapter(new MyAdapter(getChildFragmentManager()));
        tabLayout.setupWithViewPager(viewPager);
    }

    private void initViewPagerRTL() {
        viewPagerRTL = view.findViewById(R.id.viewpager_rtl);
        viewPagerRTL.setOffscreenPageLimit(tab_count);
        viewPagerRTL.setAdapter(new MyAdapter(getChildFragmentManager()));
        tabLayout.setupWithViewPager(viewPagerRTL);
    }

    public class MyAdapter extends FragmentPagerAdapter {

        public MyAdapter(FragmentManager fm) {
            super(fm, BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        }

        @NonNull
        @Override
        public Fragment getItem(int position) {

            switch (position) {
                case 0:
                    return new FragmentRadio();
                case 1:
                    return new FragmentCategory();
                case 2:
                    return new FragmentFavorite();
            }
            //noinspection ConstantConditions
            return null;
        }

        @Override
        public int getCount() {
            return tab_count;
        }

        @Override
        public CharSequence getPageTitle(int position) {
            switch (position) {
                case 0:
                    return getResources().getString(R.string.tab_recent);
                case 1:
                    return getResources().getString(R.string.tab_category);
                case 2:
                    return getResources().getString(R.string.tab_favorite);
            }

            return null;
        }
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mainActivity.setupNavigationDrawer(toolbar);
    }

    private void setupToolbar() {
        toolbar.setTitle(getString(R.string.app_name));
        mainActivity.setSupportActionBar(toolbar);
    }

    private void themeColor() {
        if (themePref.getCurrentTheme() == THEME_LIGHT) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.grey));
            tabLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorLight));
        } else if (themePref.getCurrentTheme() == THEME_DARK) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorBackgroundDark));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorToolbarDark));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.white));
            tabLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorToolbarDark));
            tabLayout.setTabTextColors((ContextCompat.getColor(getActivity(), R.color.tabUnSelectedTextColor)), (ContextCompat.getColor(getActivity(), R.color.colorDarkPrimary)));
            tabLayout.setSelectedTabIndicatorColor(ContextCompat.getColor(getActivity(), R.color.colorDarkPrimary));
        } else if (themePref.getCurrentTheme() == THEME_PRIMARY) {
            appBarLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorPrimary));
            toolbar.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorPrimary));
            toolbar.setTitleTextColor(ContextCompat.getColor(getActivity(), R.color.white));
            tabLayout.setBackgroundColor(ContextCompat.getColor(getActivity(), R.color.colorPrimary));
            tabLayout.setTabTextColors((ContextCompat.getColor(getActivity(), R.color.tabUnSelectedTextPrimaryColor)), (ContextCompat.getColor(getActivity(), R.color.white)));
            tabLayout.setSelectedTabIndicatorColor(ContextCompat.getColor(getActivity(), R.color.white));
        }
    }

}